#include <iostream>
#include <fstream>
#include <ctime>
#include <vector>
#include <cassert>
#include <map>
#include <algorithm>
#include <cmath>
#include <random>

using namespace std;

long long N, M, P;
long long F;
vector<long long> A;

void validate_input()
{
	assert(1 <= F && F <= (long long)N * M * 4 - P);
	assert(1 <= N && N <= 1000000000);
	assert(1 <= M && M <= 1000000000);
	assert(0 <= P && P <= 100000);
	sort(A.begin(), A.end());
	for (int i = 0;i < P;i++)
	{
		assert(1 <= A[i] && A[i] <= N);
		if (i != 0)
			assert(A[i-1] != A[i]);
	}
}

void solve_best()
{
	F--;
	if (P == 0)
	{		
		long long i = F / (M * 4);	//  
		cout <<i + 1LL << " " <<(F - i * M * 4LL) / 4 + 1 <<endl;
		return;
	}


	N += 2;	//  2            
	F += (long long)M * 4 - 1;	//      

	long long flats = 0;

	//        
	//     ( )
	int bp = 0, ep = A[0];

	//       .
	long long bn = 0, 
		en = 4LL * M * (ep - bp - 1) +	//   
		(4LL * M - 1) * 2 - 1;		//  -    .


	for (int i = 0;i <= P;i++)	//     
	{
		int ri, rf;	//  -   
		if (bn <= F && F <= en)	//      
		{
			int full_inps = ep - bp - 1;	// -    -   .

			//   3 
			// 1.        
			if (F < bn + M * 4 - 1)
			{
				ri = bp;
				rf = (F - bn + 1) / 4;
			}
			// 2.        
			else if (F > en - (M * 4 - 1))
			{
				ri = ep;
				rf = (F - (en - (M*4 - 1))) / 4;
			}
			// 3.  -      
			else
			{
				//       
				long long first_number = bn + M * 4 - 1;
				//   
				long long q = (F - first_number) / (M * 4) + 1;
				ri = q + bp;
				rf = (F - first_number - (q - 1) * M * 4) / 4;
			}


			cout <<ri << " " << rf + 1 <<endl;
			return;
		}

		//     
		bp = ep;
		ep = i == P - 1 ? N - 1 : A[i + 1];

		//     
		bn = en - (M * 4 - 2);
		en = bn + (ep - bp - 1) * M * 4 + 2 * M * 4 - 2 - 1;
	}

	assert(false);
}

void solve_slow()
{
	long long flats = 0;
	for (int i = 0;i < N;i++)
	{
		bool has_shop = find(A.begin(), A.end(), i + 1) != A.end();
		for (int j = 0;j < M;j++)
		{
			long long prev_flats = flats;
			if (j == 0 && has_shop)
				flats += 3;
			else
				flats += 4;

			if (prev_flats <= F && F <= flats)
			{
				cout <<i + 1 <<" " <<j + 1 <<endl;
				return;
			}
		}
	}
	assert(false);
}

std::default_random_engine generator;

long long rnd(long long f, long long t)
{
	std::uniform_int_distribution<long long> distribution(f, t);
	return distribution(generator);
}

void write_input()
{
	cout <<N <<" " <<M <<" " <<F <<" " <<P <<endl;
	if (P > 0)
	{
		for (int i = 0;i < P;i++)
			cout <<A[i] << (i == P - 1 ? "" : " ");
		cout <<endl;
	}
}

void gen_tests()
{
	int ii = RAND_MAX;

	freopen("delivery.in.gen", "w", stdout);

	generator.seed(time(NULL));

	int tc = 0;
	N = 6;
	M = 2;
	P = 0;

	//    ,    
	for (F = 1;F <= N*M*4;F++)
	{
		validate_input();
		write_input();
		tc++;
	}

	P = 1;
	A.resize(P);
	A[0] = 1;
	//      ,    
	for (F = 1;F <= N*M*4 - 1;F++)
	{
		validate_input();
		write_input();
		tc++;
	}

	A[0] = 2;
	//       ,    
	for (F = 1;F <= N*M*4 - 1;F++)
	{
		validate_input();
		write_input();
		tc++;
	}


	A[0] = 4;
	//   105     ,    
	for (F = 1;F <= N*M*4 - 1;F++)
	{
		validate_input();
		write_input();
		tc++;
	}

	A[0] = 6;
	//   105     ,    
	for (F = 1;F <= N*M*4 - 1;F++)
	{
		validate_input();
		write_input();
		tc++;
	}

	P = 2;
	A.resize(P);
	A[0] = 1;
	A[1] = 6;
	//   c  ,    
	for (F = 1;F <= N*M*4 - P;F++)
	{
		validate_input();
		write_input();
		tc++;
	}

	A[0] = 2;
	A[1] = 4;
	//   c  ,    
	for (F = 1;F <= N*M*4 - P;F++)
	{
		validate_input();
		write_input();
		tc++;
	}

	P = 6;
	A.resize(P);
	for (int i = 0;i < P;i++) A[i] = i + 1;
	//     ,    
	for (F = 1;F <= N*M*4 - P;F++)
	{
		validate_input();
		write_input();
		tc++;
	}

	N = M = 1000000000;
	P = 100000;
	A.resize(P);
	

	for (int cc = 0;cc < 10;cc++)
	{
		for (int i = 0;i < P;i++) A[i] = (i + 1) * 5000 + tc * 42;

		F = rnd(1, (long long)N*M - P);
		validate_input();
		write_input();
		tc++;
	}

	for (int cc = 0;cc < 2;cc++)
	{
		for (int i = 0;i < P;i++) A[i] = (i + 1) * 5000 + tc * 42;

		F = rnd(1, (long long)N*10);
		validate_input();
		write_input();
		tc++;
	}


	for (int cc = 0;cc < 3;cc++)
	{
		for (int i = 0;i < P;i++) A[i] = (i + 1) * 5000 + tc * 43;

		F = rnd((long long)N*M - P * 454, (long long)N*M - P);
		validate_input();
		write_input();
		tc++;
	}


	cout <<"TESTS-" <<tc;
}

int main(int argc, char *argv[])
{
	//gen_tests();
	//return 0;

	freopen("delivery.in", "r", stdin);
	//freopen("delivery.out.gen", "w", stdout);

	int tc;
	cin >>tc;


	for (int cc = 0;cc < tc;cc++)
	{	
		cin >>N >> M >> F >> P;
		A.resize(P);

		for (int i = 0;i < P;i++)
			cin >>A[i];
		sort(A.begin(), A.end());

		validate_input();
		solve_best();
		//naive_solve();
	}

	return 0;
}
